from typing import Any
import click, os
from simple_gdrive.auth import auth
from tqdm import tqdm
from loguru import logger
from simple_gdrive.config import Config, Scope
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaIoBaseDownload, MediaFileUpload
from googleapiclient.discovery import build

from simple_gdrive.fs import GPath


@click.command()
@click.option("-s", "--source", help="file path", default="")
@click.option("-i", "--fileid", help="file id", default="")
@click.option("-d", "--dest", help="output file path")
@click.option(
    "-p",
    "--permission",
    help="scope of your application",
    default=Scope.readonly.name,
    type=click.Choice([scope.name for scope in Scope]),
)
def download(source: str, fileid: str, dest: str, permission: str):
    scope = Scope[permission]
    service = build("drive", "v3", credentials=auth(scope))

    if os.path.exists(source):
        pass

    fileid = fileid.strip()
    if fileid == "":
        if source.strip() == "":
            raise Exception("source or file id is required")
        gfile = GPath.from_str(source).get_id(service)
        assert gfile is not None, f"{source} is not found"
        fileid = gfile.id
        filename = gfile.name

        if dest is None:
            dest = "./" + filename
            if os.path.exists(dest):
                raise Exception(f"destination {dest} already exists")
    else:
        if dest is None:
            raise Exception("destination is required")

    req = service.files().get_media(fileId=fileid)

    with open(dest, "wb") as fh, tqdm(total=100, desc="Download") as pbar:
        try:
            downloader = MediaIoBaseDownload(fh, req)
            done = False
            progress = 0
            while done is False:
                status, done = downloader.next_chunk()
                curr_progress = int(status.progress() * 100 * 1000) / 1000
                pbar.update(curr_progress - progress)
                progress = curr_progress

            # just in case it is not exactly 100%
            pbar.update(100 - progress)
        except HttpError as error:
            logger.exception(error)


@click.command()
@click.option("-s", "--source", help="input file")
@click.option(
    "-d",
    "--dest",
    help="the destination folder, starts with `//` to set a specific shared drive, or just `/` to use the default drive of your account",
)
@click.option("--mimetype", help="mime type")
@click.option(
    "-p",
    "--permission",
    help="scope of your application",
    default=Scope.all.name,
    type=click.Choice([scope.name for scope in Scope]),
)
def upload(source: str, dest: str, mimetype: str, permission: str):
    scope = Scope[permission]
    service = build("drive", "v3", credentials=auth(scope))

    if not os.path.exists(source):
        logger.error(f"{source} not exists")
        return

    folder_id = GPath.from_str(dest).get_id(service)

    file: Any = {"name": os.path.basename(source)}
    if folder_id is not None:
        file["parents"] = [folder_id]
    media = MediaFileUpload(source, mimetype=mimetype, resumable=True)

    req = service.files().create(
        body=file, media_body=media, fields="id", supportsAllDrives=True
    )
    resp = None
    media.stream()

    with tqdm(total=100, desc="Upload") as pbar:
        progress = 0
        while resp is None:
            status, resp = req.next_chunk()
            if status:
                curr_progress = int(status.progress() * 100 * 1000) / 1000
                pbar.update(curr_progress - progress)
                progress = curr_progress

        # just in case it is not exactly 100%
        pbar.update(100 - progress)

    logger.info(
        "\nUpload file {} to {} successfully.\nThe file is located at is: {}",
        source,
        dest,
        f"https://drive.google.com/drive/folders/{folder_id}",
    )


@click.group()
def cli():
    pass


cli.add_command(download)
cli.add_command(upload)


if __name__ == "__main__":
    cli()



import os
from typing import List, Optional
from os.path import expanduser, join
from enum import Enum


class Scope(str, Enum):
    # for all scopes, see: https://developers.google.com/identity/protocols/oauth2/scopes#drive
    all = "https://www.googleapis.com/auth/drive"
    readonly = "https://www.googleapis.com/auth/drive.readonly"


class Config:
    AuthDir: str = expanduser("~/.simple_gdrive")

    TokenFileName: str = "token.json"
    CredentialFileName: str = "credentials.json"

    @staticmethod
    def get_token_file(scope: Scope) -> str:
        return join(Config.AuthDir, scope.name, Config.TokenFileName)

    @staticmethod
    def get_credential_file(scope: Scope) -> str:
        return join(Config.AuthDir, Config.CredentialFileName)

    @staticmethod
    def get_available_scope() -> Optional[Scope]:
        for scope in [Scope.all, Scope.readonly]:
            if os.path.exists(os.path.join(Config.AuthDir, scope)):
                return scope
        return None



from genericpath import exists
import os
from pathlib import Path
from simple_gdrive.config import Config, Scope

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow


def auth(scope: Scope):
    """Authenticate with Google Drive, save the tokens to the file for future uses.

    The code is borrow from here: https://developers.google.com/drive/api/quickstart/python.

    Returns: the credentials object.
    """
    creds = None

    scopes = [scope.value]
    token_file = Config.get_token_file(scope)
    credential_file = Config.get_credential_file(scope)

    Path(token_file).parent.mkdir(exist_ok=True, parents=True)

    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists(token_file):
        creds = Credentials.from_authorized_user_file(token_file, scopes)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(credential_file, scopes)
            creds = flow.run_local_server(port=0)

        # Save the credentials for the next run
        with open(token_file, "w") as token:
            token.write(creds.to_json())

    return creds


def service():
    return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simple_gdrive']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'google-api-python-client>=2.47.0,<3.0.0',
 'google-auth-httplib2>=0.1.0,<0.2.0',
 'google-auth-oauthlib>=0.5.1,<0.6.0',
 'loguru>=0.6.0,<0.7.0',
 'tqdm>=4.64.0,<5.0.0']

entry_points = \
{'console_scripts': ['gdrive = simple_gdrive.__main__:cli']}

setup_kwargs = {
    'name': 'simple-gdrive',
    'version': '1.1.1',
    'description': 'Simple command to help download and upload big files to Google Drive (using their official Drive APIs)',
    'long_description': "**simple-gdrive**: A simple command to help download and upload big files to Google Drive (using their official APIs)\n\n### Setup\n\nBefore using the command, you must open Google Cloud Platform and create credentials for oauth: [cloud.google.com](https://cloud.google.com) > APIs & Services > Credentials. Note that if your project hasn't enabled Drive APIs, go to APIs & Services > Enabled APIs & Services and add Drive APIs.\nThen, download the JSON file and put it in the AuthDir (default at `~/.simple_gdrive`) with the name `credentials.json`. When you run the script for the first time, it will redirect to your browser so you can authenicate and give permission to YOUR application (created in the previous step) to access to your drive. Then, you will obtain an access token saving to the same drive with name `tokens.json`. Keeping the two files safe (or delete `tokens.json` after usage) and nobody else will have access to your data.\n\n### Installation\n\n```bash\npip install simple-gdrive\n```\n\n### Usage\n\nInvoke the command by running `gdrive` or `python -m simple_gdrive`.\n\nDownload a file:\n\n```bash\ngdrive download -s <path in disk> -d <path in drive>\n```\n\nUpload a file\n\n```bash\ngdrive upload -s <file path in disk> -d <path in drive>\n```\n",
    'author': 'Binh Vu',
    'author_email': 'binh@toan2.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/binh-vu/simple_gdrive',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from optparse import Option
from re import I
from typing import List, Optional
from dataclasses import dataclass


@dataclass
class GFileId:
    id: str
    name: str


@dataclass
class GPath:
    """Path of an item (file or folder) in Google Drive"""

    paths: List[str]
    drive_name: Optional[str]

    @staticmethod
    def from_str(path: str) -> "GPath":
        if path.endswith("/"):
            path = path[:-1]

        assert path.startswith(
            "/"
        ), f"Invalid path: {path}. Should starts with `/` (your drive) or `//<shared_drive_name>` (a shared drive)"

        if path.startswith("//"):
            path = path[2:]
            if path.find("/") == -1:
                drive_name = path
                paths = []
            else:
                drive_name, path = path.split("/", 1)
                paths = path.split("/")
        else:
            drive_name = None
            paths = path[1:].split("/")
        return GPath(paths, drive_name)

    def get_id(self, service) -> Optional[GFileId]:
        """Get id of the folder or file in a drive.

        Args:
            service: Google Drive service instance.
            path: Path to the file or folder.
            drive_name: Name of the shared drive.
        """
        files = service.files()
        if self.drive_name is not None:
            drive_id = self.get_drive_id(service)
            args = {
                "corpora": "drive",
                "driveId": drive_id,
                "includeItemsFromAllDrives": True,
                "supportsAllDrives": True,
            }
            parent = drive_id
            filename = self.drive_name
        else:
            args = {
                "corpora": "user",
            }
            parent = "root"
            filename = "user"

        if len(self.paths) == 0:
            if self.drive_name is None:
                return None
            else:
                return GFileId(id=parent, name=filename)

        for name in self.paths:
            resp = files.list(
                q=f"name = '{name}' and '{parent}' in parents",
                fields="files(kind, id, name, mimeType, driveId, parents)",
                pageSize=1,
                **args,
            ).execute()

            file = resp.get("files")[0]
            parent = file["id"]
            filename = file["name"]

        return GFileId(id=parent, name=filename)

    def get_drive_id(self, service):
        resp = service.drives().list().execute()
        for drive in resp.get("drives"):
            if drive["name"] == self.drive_name:
                return drive["id"]

        raise Exception(f"Drive {self.drive_name} not found")
